#include "TextEditorPanel.h"
#include "HorizontalSpacer.h"
#include "VerticalSpacer.h"

TextEditorPanel::TextEditorPanel()
    : mSaveFunction(),
      mTextBox()
{

}

void TextEditorPanel::Initialize()
{
    auto textEditorLabel = Label::Create();
    textEditorLabel->SetLabel("Text Editor");
    textEditorLabel->SetBackgroundColor(Color::Create(81, 89, 102));

    auto textEditorBox = TextBox::Create();
    mTextBox = textEditorBox.get();

    auto textEditorSaveButton = Button::Create();
    textEditorSaveButton->SetLabel("Save");
    textEditorSaveButton->SetBackgroundColor(Color::Create(156, 156, 156));
    if (mSaveFunction)
    {
        auto onSave = [this]()
                {
                    if(this && mSaveFunction && mTextBox)
                    {
                        mSaveFunction(mTextBox->GetContent());
                    };
                };
        textEditorSaveButton->SetOnClickFunction(onSave);
    }

    auto textEditorClearButton = Button::Create();
    textEditorClearButton->SetLabel("Clear");
    textEditorClearButton->SetBackgroundColor(Color::Create(156, 156, 156));
    auto buttonSpacer = VerticalSpacer::Create();
    buttonSpacer->AddWidgetWithScale(std::move(textEditorSaveButton), 1);
    buttonSpacer->AddWidgetWithScale(std::move(textEditorClearButton), 1);

    auto hSpacer = HorizontalSpacer::Create();
    hSpacer->AddWidgetWithScale(std::move(textEditorLabel), 1);
    hSpacer->AddWidgetWithScale(std::move(textEditorBox), 8);
    hSpacer->AddWidgetWithScale(std::move(buttonSpacer), 1);

    AddWidget(std::move(hSpacer));
}

std::unique_ptr<TextEditorPanel> TextEditorPanel::Create()
{
    return std::make_unique<TextEditorPanel>();
}

void TextEditorPanel::SetOnSaveFunction(saveFunc func)
{
    mSaveFunction = func;
}
