#include "Button.h"
#include <iostream>

Button::Button()
    : Widget(),
      mLabel(),
      mClickFunc()
{

}

std::unique_ptr<Button> Button::Create()
{
    return std::make_unique<Button>();
}

void Button::SetOnClickFunction(clickFunc func)
{
    mClickFunc = func;
}

void Button::SetLabel(const std::string& text)
{
    mLabel = text;
}

void Button::OnMyMouseEvent(const MouseEvent* event)
{
    if(event->GetAction() == MouseEvent::Action::Pressed)
    {
        SetBackgroundColor(Color::Create(0, 255, 0));
        if(mClickFunc)
        {
            mClickFunc();
        }
    }
    else if(event->GetAction() == MouseEvent::Action::Released)
    {
        SetBackgroundColor(Color::Create(0, 255, 255));
    }
}

void Button::OnPaintEvent(const PaintEvent* event)
{
    mMyLayers.clear();
    AddBackground(event);
    if(!mLabel.empty())
    {
        auto middle = DiscretePoint(mLocation.GetX() + mWidth/2,
                mLocation.GetY() + mHeight/2);
        auto textLayer = VisualLayer::Create();
        textLayer->SetText(TextElement::Create(mLabel, middle));
        mMyLayers.push_back(std::move(textLayer));
    }
    CopyMyLayers();
}
