#include "HorizontalSpacer.h"

#include <algorithm>
#include <numeric>

HorizontalSpacer::HorizontalSpacer()
    : Widget(),
      mScales()
{

}

std::unique_ptr<HorizontalSpacer> HorizontalSpacer::Create()
{
    return std::make_unique<HorizontalSpacer>();
}

void HorizontalSpacer::AddWidget(WidgetUPtr widget)
{
    AddWidgetWithScale(std::move(widget), 1.0);
}

void HorizontalSpacer::AddWidgetWithScale(WidgetUPtr widget, double scale)
{
    Widget::AddWidget(std::move(widget));
    mScales.push_back(scale);
}

void HorizontalSpacer::AddChildLayers(const PaintEvent* event)
{
    mLayers.clear();
    double scaleSum  = std::accumulate(mScales.begin(), mScales.end(), 0.0);
    double offset = 0;
    double height = mHeight;
    if (mMaxHeight > 0 && height > mMaxHeight)
    {
        height = mMaxHeight;
    }
    for(std::size_t idx=0; idx<mChildren.size(); idx++)
    {
        auto& child = mChildren[idx];
        double scale = mScales[idx];
        double delta = height * (scale/scaleSum);
        if (child->GetMaxHeight() > 0 && delta > child->GetMaxHeight())
        {
            delta = child->GetMaxHeight();
        }
        child->SetSize(mWidth, delta);
        child->SetLocation(DiscretePoint(mLocation.GetX(), mLocation.GetY() + offset));
        child->OnPaintEvent(event);
        auto layers = child->GetLayers();
        mLayers.insert(mLayers.end(), layers.begin(), layers.end());
        offset += delta;
    }
}

void HorizontalSpacer::OnPaintEvent(const PaintEvent* event)
{
    AddChildLayers(event);
}
